<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Services_Naver.
 *
 * This is the main file of the Services_Naver package. This file has to be
 * included for usage of Services_Naver.
 *
 * PHP versions 4 and 5
 *
 * license here
 *
 * @category   Webservices
 * @package    Naver
 * @author     Sang-hyun, Park <bs0048@teramail.com>
 * @copyright
 * @license    PHP License
 * @version
 * @link       http://phpnaverapi.kldp.net/
 * @since
 */
require_once 'XML/RSS.php';

define('SERVICES_NAVER_ENCODING', 'euckr');

class Services_Naver {

    /**
     * required user key
     *
     * @var string
     * @since 0.1.0
     * @access private
     */
    var $_key;


    var $_url = 'http://openapi.naver.com/search';

    /**
     * Client side encoding for auto-en/decoding
     *
     * @var string
     * @access private
     */
    var $_encoding = SERVICES_NAVER_ENCODING;

    /**
     * Constructor 
     * 
     *  
     * @since 0.1.0
     * @param  string  $key       required key
     * @param  string  $encoding  client side encoding
     * @access protected
     * @return void
     */
    function Services_Naver ($key = null, $encoding = null) {
        Services_Naver::__constructor($key, $encoding);
    }

    /**
     * *REAL* Constructor (for php5)
     *
     * @see also Services_Naver()
     */
    function __constructor($key = null, $encoding = null) {
        if (null != $key) {
            $this->setKey($key);
        }
        $this->setEncoding( $encoding != null
                                ? $encoding
                                : SERVICES_NAVER_ENCODING );
    }

    /**
     *
     * @access public
     * @return void
     */
    function setKey($key) {
        $this->_key = $key;
    }

    /**
     * set client side encoding for auth-en/decoding and create converting function
     *
     * @access public
     * @param  string  $encoding
     * @return void
     */
    function setEncoding($encoding) {
        $this->_encoding    = $encoding;
        $this->_decode_func = create_function('$str', 'return mb_convert_encoding($str, "' . $this->_encoding . '", "utf-8");');
        $this->_encode_func = create_function('$str', 'return mb_convert_encoding($str, "utf-8", "' . $this->_encoding . '");');
    }

    /**
     * send search request to Naver
     *
     * @param  string  target
     * @param  string  query
     * @param  int     display  num of results to fetch
     * @param  int     start    search start position
     * @param  string  sort
     * @access public
     *
     */
    function _query($target, $query, $display = null, $start = null, $sort = null) {
        if (strlen($this->_key) == 0) {
            return Pear::raiseError('give me your key');
        }
        $target = strtolower($target);
        $this->_last_query = $query;

        switch($target) {
            case 'kin':    // iN
            case 'blog':   // α
            case 'doc':    // 
            case 'webkr':  // 
            case 'book':   // å
                // isbn, price, discount, imageurl, publisher, pubdate
            case 'shop':   // 
                // price, image
            case 'krdic':  // 
            case 'jrdic':  // Ͼ
            case 'endic':  // 
                @include_once 'HTTP/Request.php';
                @include_once 'XML/RSS.php';

                $req = new HTTP_Request($this->_url);
                $req->addQueryString('key',     $this->_key);
                $req->addQueryString('target',  $target);
                $req->addQueryString('query',   mb_convert_encoding($query, 'utf-8', $this->_encoding));
                $req->addQueryString('display', $display);
                $req->addQueryString('start',   $start);
                $req->addQueryString('sort',    $sort);
                $req->sendRequest();
                switch ($req->getResponseCode()) {
                    case '410':
                        return Pear::raiseError('maximum query over');
                        break;
                    case '404':
                        return Pear::raiseError('404 not found');
                        break;
                    case 200:
                        break;
                    default:
                        return Pear::raiseError('Unspecified http state return');
                        break;
                }

                $rss = &new XML_RSS();
                $rss->setInputString($req->getResponseBody());
                if (Pear::isError($r = $rss->parse())) {
                    echo $r->getMessage();
                    return Pear::raiseError('');
                }
                $result = $rss->getItems();
                return $this->_arrayMapRecursive($this->_decode_func, $result);
                break;
            case 'rank':   // ǽð ˻
            case 'recmd':  // õ˻
            case 'adult':  // ΰ˻Ǻ
            case 'errata': // Ÿȯ
                //  ̹ ü  
                break;
            case 'j': // ٷΰ
                // http redirect  Ȯغ  meta±׸ ̿ϴ°?
                // ׸ ̰͸ request url search.naver.com/search.naver  -_-;;
                // key Ѱ ʾƵ ǰ, where   ʿϰ, encoding̶  ִ -_-;;
                // ٸ ޽ ó ؾ߰ڴ
                break;
        }
    }

    /**
     * Recursively applies the callback function to the value 
     * this function from PEAR::HTTP_Request
     * 
     * @param    mixed   Callback function
     * @param    mixed   Value to process
     * @access   private
     * @return   mixed   Processed value
     */
    function _arrayMapRecursive($callback, $value)
    {
        if (!is_array($value)) {
            return call_user_func($callback, $value);
        } else {
            $map = array();
            foreach ($value as $k => $v) {
                $map[$k] = $this->_arrayMapRecursive($callback, $v);
            }
            return $map;
        }
    }
}
